/** @module 类型判断-type */
/**
 * 判断是否是数组
 *
 * @param {*} array - 需要判断的对象
 */
function isArray(array) {
    return Object.prototype.toString.call(array) == '[object Array]';
}

/** @module 字符串相关-string */
/**
 * 以某字符分隔字符串
 * @example
 * ```js
 * divideText(10000000000000, 3, ',') => 10,000,000,000,000
 * ```
 *
 * @param {string/number} text - 需要被分割的文本
 * @param {number} len - 分割长度
 * @param {string} [replaceText=','] - 用来替换的字符
 * @returns {string}
 */
function divideText(text, len, replaceText) {
    if (replaceText === void 0) { replaceText = ','; }
    var reg = new RegExp('(\\d)(?=(?:\\d{' + +len + '})+$)', 'g');
    return ("" + text).replace(reg, '$1' + replaceText);
}
/**
 * 隐藏/替换字符串中间几位
 * @example
 * ```js
 * spliceText(18255558888, 3, 4, '****') // 182\*\*\*\*8888
 * ```
 *
 * @param {string/number} text - 需要被替换的文本
 * @param {number} start - start从前往后第几位
 * @param {number} last - end从后往前第几位
 * @param {string} replaceText - 用来替换的字符
 * @returns {string}
 */
function spliceText(text, start, last, replaceText) {
    var reg = new RegExp('^(.{' + +start + '})(?:\\d+)(.{' + +last + '})$');
    return ("" + text).replace(reg, '$1' + replaceText + '$2');
}
/**
 * 金额每三位正数添加逗号，支持保留小数
 * @example
 * ```js
 * toDecimal(10000000000000, 2) // 10,000,000,000.00
 * ```
 *
 * @param {string/number} textt - 需要被替换的文本
 * @param {number} len - 保留小数位数
 * @returns {string}
 */
function toDecimal(text, len) {
    text = text.toString();
    if (/[^0-9.]/.test(text) || text == null || text == '')
        { return ''; }
    text = text.replace(/^(\d*)$/, '$1.');
    text = (text + '00').replace(/(\d*\.\d\d)\d*/, '$1');
    text = text.replace('.', ',');
    var re = /(\d)(\d{3},)/;
    while (re.test(text))
        { text = text.replace(re, '$1,$2'); }
    text = text.replace(/,(\d\d)$/, '.$1');
    if (len == 0) {
        // 不带小数位(默认是有小数位)
        var a = text.split('.');
        if (a[1] == '00') {
            text = a[0];
        }
    }
    return text;
}

/** @module 字符串相关-string */
/**
 * 根据对象数组的某字段进项排序
 *
 * @param {any[]} array - 需要排序的数组，其中每一项是对象
 * @param {string} key - 排序的字段
 * @param {boolean} [flag=false] - 是否需要倒序， 默认false正序，true则为倒序
 * @returns {string} - 排序后的数组
 */
function sortJSON(array, key, flag) {
    if (flag === void 0) { flag = false; }
    var sortBy = function (filed, flag, primer) {
        var flagIndex = flag ? -1 : 1;
        return function (a, b) {
            var a1 = +a[filed];
            var b1 = +b[filed];
            if (typeof primer != 'undefined') {
                a1 = primer("" + a1);
                b1 = primer("" + b1);
            }
            if (a1 < b1) {
                return flagIndex * -1;
            }
            if (a1 > b1) {
                return flagIndex * 1;
            }
            return 1;
        };
    };
    return array.sort(sortBy(key, flag, parseInt));
}

/** @module 字符串相关-string */
/**
 * 驼峰命名转换为短横线命名
 *
 * @param {string} text - 需要转换的驼峰命名的字符串
 * @returns {string}
 */
function camelToKebab(text) {
    return text.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
}
/**
 * 短横线命名转换为驼峰命名
 *
 * @param {string} text - 需要转换的短横线命名的字符串
 * @returns { string }
 */
function kebabToCamel(text) {
    return text.replace(/-([a-z])/g, function (g) {
        return g[1].toUpperCase();
    });
}

/** @module 字符串相关-string */
/**
 *  过滤文本中的html标签
 *
 * @param {string} str - 文本片段
 * @returns {string}
 */
function formatHTML(str) {
    str = str.replace(/<style([\s\S]*?)<\/style>/gi, '');
    str = str.replace(/<script([\s\S]*?)<\/script>/gi, '');
    str = str.replace(/<(div|p)[^>]*>/gi, '\n');
    str = str.replace(/<\/li>/gi, '\n');
    str = str.replace(/<li>/gi, ' * ');
    str = str.replace(/<\/ul>/gi, '\n');
    str = str.replace(/<\/p>/gi, '\n');
    str = str.replace(/<br\s*\/?>/gi, '\n');
    str = str.replace(/^\n*|<[^>]+>|\n*$/gi, '');
    str = str.replace(/<[^>]+>/gi, '');
    str = str.replace(/\n{2,}/g, '\n\n');
    str = str.replace(/\n+$/g, '');
    str = str.replace(/&nbsp;/gi, ' ');
    return str;
}

/** @module 字符串相关-string */
/**
 * 随机生成长度为len的字符串
 *
 * @param {number} len - 生成字符串的长度
 * @returns {string}
 */
function randomString(len) {
    return Math.random()
        .toString(36)
        .substring(2, len + 2);
}

/**
 * 获取高亮分词列表
 * @param {(string/Regexp)} keyword - 关键字
 * @param {string} fulltext - 整段文本
 * @returns {object[]}
 */
function getHighlightList(keyword, fulltext) {
    var reg = keyword instanceof RegExp ? keyword : new RegExp(keyword, 'ig');
    var list = [];
    var item = null;
    var prevIndex = 0;
    while ((item = reg.exec(fulltext)) !== null) {
        var curr = fulltext.substring(prevIndex, item.index);
        curr && list.push({ text: curr });
        list.push({ text: item[0], highlight: true });
        prevIndex = item.index + item[0].length;
    }
    list.push({ text: fulltext.substring(prevIndex) });
    return list;
}
/**
 * 获取高亮分词模板
 *
 * @param {(string/Regexp)} keyword - 关键字
 * @param {string} fulltext - 整段文本
 * @param {string} [tag=span] - 高亮文本标签
 * @param {string} [classname=highlight] - 高亮文本标签类名
 * @returns {string} tag标签包裹的字符串
 */
function getHighlightTemplate(keyword, fulltext, tag, classname) {
    if (tag === void 0) { tag = 'span'; }
    if (classname === void 0) { classname = 'highlight'; }
    var reg = keyword instanceof RegExp ? keyword : new RegExp(keyword, 'ig');
    var item = null;
    var prevIndex = 0;
    var str = '';
    while ((item = reg.exec(fulltext)) !== null) {
        var curr = fulltext.substring(prevIndex, item.index);
        str = curr ? "" + str + curr : str;
        str = str + "<" + tag + " class=\"" + classname + "\">" + item[0] + "</" + tag + ">";
        prevIndex = item.index + item[0].length;
    }
    str = "" + str + fulltext.substring(prevIndex);
    return str;
}
/**
 * 获取高亮分词信息
 *
 * @param {(string/Regexp)} keyword - 关键字
 * @param {string} fulltext - 整段文本
 * @param {string} [tag] - 高亮文本标签,具有tag参数，则返回该标签包裹的字符串,否则返回数组对象
 * @param {string} [classname] - 高亮文本标签类名
 * @returns {string | Object[]}
 */
function getHighlight(keyword, fulltext, tag, classname) {
    return tag ? getHighlightTemplate(keyword, fulltext, tag, classname) : getHighlightList(keyword, fulltext);
}

/** @module 数组-array */
/**
 * 求数组并集
 *
 * @param {Array} a
 * @param {Array} b
 */
function unionArr(a, b) {
    return Array.from(new Set(a.concat(b)));
}
/**
 * 求数组交集
 *
 * @param {Array} a
 * @param {Array} b
 */
function intersectArr(a, b) {
    return Array.from(new Set(a.slice().filter(function (x) { return b.includes(x); })));
}
/**
 * 求数组差集
 *
 * @param {Array} a
 * @param {Array} b
 */
function differenceArr(a, b) {
    return Array.from(new Set(a.slice().filter(function (x) { return !b.includes(x); })));
}

/** @module 数组-array */
/**
 * 基本类型数组去重
 *
 * @param {Array} arr - 需要去重的数组
 */
function uniqueArr(arr) {
    return Array.from(new Set(arr));
}
/**
 * 对象数组单个键值匹配去重
 *
 * @param {Array} arr - 需要去重的数组
 * @param {String} key - 单个键值
 */
function uniqueKeyInArr(arr, key) {
    var cache = {};
    return arr.reduceRight(function (item, next) {
        if (!cache[next[key]]) {
            cache[next[key]] = true;
            item.push(next);
        }
        return item;
    }, []);
}
/**
 * 对象数组完全匹配去重
 *
 * @param {Array} arr - 需要去重的数组
 */
function uniqueObjInArr(arr) {
    if (!isArray(arr))
        { return arr; }
    return Array.from(new Set(arr.map(function (item) { return JSON.stringify(item); }))).map(function (v) { return JSON.parse(v); });
}

/** @module 数组-array */
/**
 * 规整多重数组
 *
 * @param {Array} arr - 需要规整的数组
 */
function flattenArr(arr) {
    if (!isArray(arr))
        { return arr; }
    return arr.reduce(function (done, curr) { return done.concat(flattenArr(curr)); }, []);
}

/** @module 存储相关-storage */
/**
 * localStorage获取值
 *
 * @param {string} key - 存储的键
 * @param {number} [day] - 有效天数
 */
function getStorage(key, day) {
    var dateStr = localStorage.getItem(key);
    if (!dateStr)
        { return ''; }
    if (!day)
        { return dateStr; }
    var obj = JSON.parse(dateStr);
    return new Date().getTime() - Number(obj.date) > 86400000 * day ? '' : obj.value;
}
/**
 * localStorage设置值
 *
 * @param {string} key - 存储的键
 * @param {any} value - 存储的值
 * @param {number} [day] - 有效天数
 */
function setStorage(key, value, day) {
    if (!day) {
        localStorage.setItem(key, value);
    }
    else {
        var params = {
            date: new Date().getTime(),
            value: value
        };
        localStorage.setItem(key, JSON.stringify(params));
    }
}
/**
 * localStorage移除键为key的存储
 *
 * @param {string} key - 存储的键
 */
function removeStorage(key) {
    localStorage.removeItem(key);
}
/**
 * localStorage清空存储
 */
function clearStorage() {
    localStorage.clear();
}

/** @module 存储相关-storage */
/**
 * 设置Cookie
 *
 * @param {string} key - 存储的键
 * @param {any} value - 存储的值
 * @param {number} [expiredays] - 有效天数
 * @param {string} [domain] - 共享Cookie域名
 */
function setCookie(key, value, expiredays, domain) {
    value = encodeURIComponent(value);
    var exdate = new Date();
    if (expiredays) {
        exdate.setDate(exdate.getDate() + expiredays);
    }
    var exdatestr = exdate.toUTCString();
    document.cookie = domain
        ? key + "=" + value + ";expires=" + exdatestr + ";path=/;domain=" + domain
        : key + "=" + value + ";expires=" + exdatestr + ";path=/;";
}
/**
 * 获取Cookie
 *
 * @param key
 */
function getCookie(key) {
    var reg = new RegExp("(^| )" + key + "=([^;]*)(;|$)");
    var arr = document.cookie.match(reg);
    return arr ? decodeURIComponent(arr[2]) : null;
}
/**
 * 清除Cookie
 *
 * @param {string} key - 存储的键
 * @param {string} [domain] - 共享Cookie域名
 */
function removeCookie(key, domain) {
    setCookie('', key, -1, domain);
}

/** @module Map、Set相关-map */
/**
 *  Map转换成Object
 *
 * @param {Map} map - map对象
 * @returns {Object}
 */
function mapToObj(map) {
    var obj = Object.create(null);
    for (var _i = 0, map_1 = map; _i < map_1.length; _i++) {
        var _a = map_1[_i], k = _a[0], v = _a[1];
        obj[k] = v;
    }
    return obj;
}
/**
 *  Object转换成Map
 *
 * @param {Object} obj - map对象
 * @returns {Map}
 */
function objToMap(obj) {
    var map = new Map();
    for (var _i = 0, _a = Object.keys(obj); _i < _a.length; _i++) {
        var k = _a[_i];
        map.set(k, obj[k]);
    }
    return map;
}

/** @module 时间-date */
/**
 * 返回格林威治时间
 *
 * @example
 * ```js
 * getDate()
 * // or
 * getDate('2019-08-24')
 * // or
 * getDate(1566629188014)
 * // or
 * getDate(date)  // date => Sat Aug 24 2019 14:47:16 GMT+0800 (中国标准时间)
 * ```
 *
 * @param {number/string/Date/undefined} [date=''] - 数字、字符串、格林威治时间、空值
 * @returns {Date}
 */
function getDate(date) {
    if (date === void 0) { date = ''; }
    if (!date)
        { return new Date(); }
    if (date instanceof Date)
        { return date; }
    if (typeof date === 'number')
        { return new Date(date); }
    if (typeof date === 'string') {
        date = date.replace(/-/gi, '/');
        return new Date(date);
    }
    return new Date(date);
}
/**
 * 前缀加0
 * @example
 * ```js
 * prefixDate(7) // 07
 * // or
 * prefixDate('7') // 07
 * ```
 *
 * @param {number/string} time - 需要加前缀的数字或字符串
 * @returns {string}
 */
function prefixDate(time) {
    return time < 10 ? "0" + time : "" + time;
}
/**
 * 自定义时间转换
 *
 * @example
 * ```js
 * formatDate() // 2019-08-24 16:08
 * // or
 * formatDate(1566629188014, 'YYYY-MM') // 2019-08
 * // or
 * formatDate(new Date(), 'week') // 六
 * ```
 *
 *
 * @param {number/string/Date/undefined} [date=''] - 数字、字符串、格林威治时间、空值
 * @param {string} [format='YYYY-MM-DD HH:mm'] - 转换格式（由YYYY、YY、MM、DD、HH、hh、mm、SS、ss、week组合而成）
 * @returns {string}
 */
function formatDate(date, format) {
    if (date === void 0) { date = ''; }
    if (format === void 0) { format = 'YYYY-MM-DD HH:mm'; }
    var dateTime = getDate(date);
    var week = ['日', '一', '二', '三', '四', '五', '六'];
    return format.replace(/YYYY|YY|MM|DD|HH|hh|mm|SS|ss|week/g, function (key) {
        switch (key) {
            case 'YYYY':
                return "" + dateTime.getFullYear();
            case 'YY':
                return (dateTime.getFullYear() + '').slice(2);
            case 'MM':
                return prefixDate(dateTime.getMonth() + 1);
            case 'DD':
                return prefixDate(dateTime.getDate());
            case 'HH':
            case 'hh':
                return prefixDate(dateTime.getHours());
            case 'mm':
                return prefixDate(dateTime.getMinutes());
            case 'SS':
            case 'ss':
                return prefixDate(dateTime.getSeconds());
            case 'week':
                return week[dateTime.getDay()];
            default:
                throw new Error('format error');
        }
    });
}
/**
 * 获取N天后的日期
 *
 * @example
 * ```js
 * getNextDate(3) // 2019-08-27T08:18:53.212Z
 * // or
 * getNextDate(2, 1566629188014, 'YYYY-MM--DD') // 2019-08--26
 * ```
 *
 * @param {number} day - 天数
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getNextDate(day, currDate, format) {
    if (currDate === void 0) { currDate = ''; }
    var next = getDate(currDate);
    next.setDate(next.getDate() + day);
    return format ? formatDate(next, format) : next;
}
/**
 * 获取某个月有多少天
 *
 * @example
 * ```js
 * getMonthDays() // 31
 * // or
 * getMonthDays(1566629188014) // 31
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @returns {number}
 */
function getMonthDays(currDate) {
    if (currDate === void 0) { currDate = ''; }
    var dd = getDate(currDate);
    dd.setMonth(dd.getMonth() + 1, 0);
    return dd.getDate();
}
/**
 * 获取上个月初时间
 * @example
 * ```js
 * getPrevMonth() // 2019-07-01T08:32:36.800Z
 * // or
 * getPrevMonth('', 'YYYY-MM-DD') // 2019-07-01
 * // or
 * getPrevMonth('2019-01-19', 'YYYY-MM-DD') // 2018-12-01
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getPrevMonth(currTime, format) {
    if (currTime === void 0) { currTime = ''; }
    var prev = getDate(currTime);
    prev.setDate(0);
    prev.setDate(1);
    return format ? formatDate(prev, format) : prev;
}
/**
 * 获取下个月初时间
 * @example
 * ```js
 * getNextMonth() // 2019-09-01T08:41:32.891Z
 * // or
 * getNextMonth('', 'YYYY-MM-DD') // 2019-09-01
 * // or
 * getNextMonth('2019-01-19', 'YYYY-MM-DD') // 2019-02-01
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getNextMonth(currTime, format) {
    if (currTime === void 0) { currTime = ''; }
    var prev = getDate(currTime);
    prev.setDate(0);
    prev.setDate(1);
    return format ? formatDate(prev, format) : prev;
}
/**
 * 获取倒计时时间
 * @example
 * ```js
 * getCountDown(1566662400000) // 4小时31分钟30秒
 * // or
 * getCountDown('2019-08-25') // 4小时31分钟30秒
 * ```
 *
 * @param {number/string/Date/undefined} nextDate - 到期时间
 * @param {object} options - 参数配置
 * @param {string} options.day - 天数配置语
 * @param {string} options.hour - 小时配置语
 * @param {string} options.minute - 分钟配置语
 * @param {string} options.second - 秒配置语
 * @param {string} options.end - 刚到时间配置语
 * @param {string} options.over - 已过期配置语
 * @returns {string}
 */
function getCountDown(nextDate, options) {
    if (options === void 0) { options = {
        day: '天',
        hour: '小时',
        minute: '分钟',
        second: '秒',
        end: '时间到',
        over: '已过期'
    }; }
    var distance = getDate(nextDate).getTime() - Date.now();
    if (distance <= 0 && distance > -1000)
        { return options.end; }
    if (distance <= -1000)
        { return options.over; }
    var days = Math.floor(distance / 86400000);
    var hours = Math.floor((distance % 86400000) / 3600000);
    var minutes = Math.floor((distance % 3600000) / 60000);
    var seconds = Math.floor((distance % 60000) / 1000);
    if (days > 0) {
        return "" + days + options.day + hours + options.hour + minutes + options.minute + seconds + options.second;
    }
    else if (hours > 0) {
        return "" + hours + options.hour + minutes + options.minute + seconds + options.second;
    }
    else if (minutes > 0) {
        return "" + minutes + options.minute + seconds + options.second;
    }
    else {
        return "" + seconds + options.second;
    }
}
/**
 * 获取N天前、N小时、N分钟前
 * @example
 * ```js
 * getAgoDate(1566662400000)  // 刚刚
 * // or
 * getAgoDate('2019-08-24', 'YYYY-MM-DD')   // 23小时前
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照时间
 * @param {string} [format='YYYY-MM-DD'] - 转换格式，有format返回string
 * @param {object} options - 参数配置
 * @param {string} options.day - 天数配置语
 * @param {string} options.hour - 小时配置语
 * @param {string} options.minute - 分钟配置语
 * @param {string} options.second - 秒配置语
 * @param {string} options.just - 刚刚配置语
 * @returns {number}
 */
function getAgoDate(currDate, format, options) {
    if (currDate === void 0) { currDate = ''; }
    if (format === void 0) { format = 'YYYY-MM-DD'; }
    if (options === void 0) { options = {
        day: '天前',
        hour: '小时前',
        minute: '分钟前',
        second: '秒前',
        just: '刚刚'
    }; }
    var getTodayUnix = function () {
        var date = getDate();
        date.setHours(0, 0, 0, 0);
        return date.getTime();
    };
    var now = getDate().getTime();
    var today = getTodayUnix();
    var timestamp = getDate(currDate).getTime();
    var timer = (now - timestamp) / 1000;
    var tip = '';
    if (timer <= 0 || Math.floor(timer / 60) <= 0) {
        tip = options.just;
    }
    else if (timer < 3600) {
        tip = Math.floor(timer / 60) + ("" + options.minute);
    }
    else if (timer >= 3600 && timestamp - today >= 0) {
        tip = Math.floor(timer / 3600) + ("" + options.hour);
    }
    else if (timer / 86400 <= 7) {
        tip = Math.ceil(timer / 86400) + ("" + options.day);
    }
    else {
        tip = formatDate(timestamp, format);
    }
    return tip;
}
/**
 * 获取某月最后一天
 *
 * @example
 * ```js
 * getMonthLastDay(1566662400000) // 2019-08-30T16:00:00.000Z
 * //or
 * getMonthLastDay('2019-08-25') // 2019-08-30T16:00:00.000Z
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getMonthLastDay(currDate, format) {
    if (currDate === void 0) { currDate = ''; }
    var next = getDate(currDate);
    next.setMonth(next.getMonth() + 1, 0);
    return format ? formatDate(next, format) : next;
}
/**
 * 获取某周的第一天，以周日计算
 *
 * @example
 * ```js
 * getWeekFirstDay(1566662400000) // 2019-08-24T16:00:00.000Z
 * // or
 * getWeekFirstDay('2019-08-25') // 2019-08-24T16:00:00.000Z
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getWeekFirstDay(currDate, format) {
    if (currDate === void 0) { currDate = ''; }
    var first = getDate(currDate);
    first.setDate(first.getDate() - first.getDay());
    return format ? formatDate(first, format) : first;
}
/**
 * 获取某周的最后一天，以周六计算
 *
 * @example
 * ```js
 * getWeekFirstDay(1566662400000)  // Sun Aug 25 2019 00:00:00 GMT+0800 (中国标准时间)
 * // or
 * getWeekFirstDay('2019-08-24', 'YYYY-MM-DD')   // 2019-08-18
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getWeekLastDay(currDate, format) {
    if (currDate === void 0) { currDate = ''; }
    var last = getDate(currDate);
    last.setDate(6 + last.getDate() - last.getDay());
    return format ? formatDate(last, format) : last;
}
/**
 * 获取某月第一个周一
 * @example
 * ```js
 * getFirstMonday(1566662400000)  // Mon Jul 29 2019 00:00:00 GMT+0800 (中国标准时间)
 * // or
 * getFirstMonday('2019-08-24', 'YYYY-MM-DD')   // 2019-07-29
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getFirstMonday(currDate, format) {
    if (currDate === void 0) { currDate = ''; }
    var date = getDate(currDate);
    date.setDate(1);
    var firstIndex = date.getDay();
    firstIndex = firstIndex == 0 ? -5 : 2 - firstIndex;
    date.setDate(firstIndex);
    date.setHours(0, 0, 0, 0);
    return format ? formatDate(date, format) : date;
}
/**
 * 获取某月最后一个周日
 * @example
 * ```js
 * getLastSunday(1566662400000)  // Sun Sep 01 2019 00:00:00 GMT+0800 (中国标准时间)
 * // or
 * getLastSunday('2019-08-24', 'YYYY-MM-DD')   // 2019-09-01
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getLastSunday(currDate, format) {
    if (currDate === void 0) { currDate = ''; }
    var last = getDate(currDate);
    last.setMonth(last.getMonth() + 1, 0);
    var lastIndex = last.getDay();
    lastIndex = lastIndex == 0 ? 0 : 7 - lastIndex;
    last.setDate(last.getDate() + lastIndex);
    last.setHours(0, 0, 0, 0);
    return format ? formatDate(last, format) : last;
}
/**
 * 获取某月第一个周日
 * @example
 * ```js
 * getFirstSunday(1566662400000)  // Sun Jul 28 2019 00:00:00 GMT+0800 (中国标准时间)
 * // or
 * getFirstSunday('2019-08-24', 'YYYY-MM-DD')   // 2019-07-28
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getFirstSunday(currDate, format) {
    if (currDate === void 0) { currDate = ''; }
    var last = getDate(currDate);
    last.setMonth(last.getMonth() + 1, 0);
    var lastIndex = last.getDay();
    lastIndex = lastIndex == 0 ? 0 : 7 - lastIndex;
    last.setDate(last.getDate() + lastIndex);
    last.setHours(0, 0, 0, 0);
    return format ? formatDate(last, format) : last;
}
/**
 * 获取某月最后一个周六
 * @example
 * ```js
 * getLastSaturday(1566662400000)  // Sat Aug 31 2019 00:00:00 GMT+0800 (中国标准时间)
 * // or
 * getLastSaturday('2019-08-24', 'YYYY-MM-DD')   // 2019-08-31
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @param {string} [format] - 转换格式，有format返回string
 * @returns {string/Date}
 */
function getLastSaturday(currDate, format) {
    if (currDate === void 0) { currDate = ''; }
    var date = getDate(currDate);
    date.setMonth(date.getMonth() + 1, 0);
    date.setDate(6 + date.getDate() - date.getDay());
    date.setHours(0, 0, 0, 0);
    return format ? formatDate(date, format) : date;
}
/**
 * 获取日期今年第几周，以周一开始算
 * @example
 * ```js
 * getWeekIndex(1566662400000)  // 35
 * // or
 * getWeekIndex('2019-08-24', 'YYYY-MM-DD')   // 35
 * ```
 *
 * @param {number/string/Date/undefined} [currDate=''] - 参照初始时间
 * @returns {number}
 */
function getWeekIndex(currDate) {
    if (currDate === void 0) { currDate = ''; }
    var now = getDate(currDate).getTime();
    var firstDay = getDate(currDate);
    firstDay.setMonth(0);
    firstDay.setDate(1);
    var num = firstDay.getDay();
    num = 7 - num;
    var rq = now - firstDay.getTime();
    return Math.ceil((Math.ceil(rq / (24 * 60 * 60 * 1000)) + num) / 7);
}

/** @module 浏览器相关-browser */
/**
 * 是否具有 className
 *
 * @param {HTMLElement} el - 元素
 * @param {string} cls - 类名
 */
function hasClass(el, cls) {
    return el.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'));
}
/**
 * 添加 className
 *
 * @param {HTMLElement} el - 元素
 * @param {string} cls - 类名
 */
function addClass(el, cls) {
    if (hasClass(el, cls))
        { return; }
    el.className += el.className ? " " + cls : cls;
}
/**
 * 删除 className
 *
 * @param {HTMLElement} el - 元素
 * @param {string} cls - 类名
 */
function removeClass(el, cls) {
    if (!hasClass(el, cls))
        { return; }
    var reg = new RegExp('(\\s|^)' + cls + '(\\s|$)');
    el.className = el.className.replace(reg, ' ');
}
/**
 * 切换 className
 *
 * @param {HTMLElement} el - 元素
 * @param {string} cls - 类名
 */
function toggleClass(el, cls) {
    return !hasClass(el, cls) ? addClass(el, cls) : removeClass(el, cls);
}

/** @module 浏览器相关-browser */
/**
 * 获取链接中键相应的值
 *
 * @param {string} name - 键名
 * @returns {string}
 */
function getUrlParam(name) {
    var reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i');
    var r = window.location.search.substr(1).match(reg);
    return r ? decodeURIComponent(r[2]) : null;
}
/**
 * 返回链接中所有键值对象
 *  @returns {Object}
 */
function getUrlObj() {
    var qs = location.search.length > 0 ? location.search.substring(1) : '';
    var args = {};
    var items = qs.length ? qs.split('&') : [];
    var item = null;
    var name = null;
    var value = null;
    var i = 0;
    var len = items.length;
    for (i = 0; i < len; i++) {
        item = items[i].split('=');
        name = decodeURIComponent(item[0]);
        value = decodeURIComponent(item[1]);
        if (name.length) {
            args[name] = value;
        }
    }
    return args;
}
/**
 *
 * 向链接中添加键值对
 *
 * @param {string} url - 链接
 * @param {string} name - 键
 * @param {any} value - 值
 * @returns {string}
 */
function addUrlParam(url, name, value) {
    url += url.indexOf('?') == -1 ? '?' : '&';
    url += encodeURIComponent(name) + '=' + encodeURIComponent(value);
    return url;
}
/**
 * 修改链接中键对应的值
 *
 * @param {string} url - 链接
 * @param {string} name - 键
 * @param {any} value - 值
 * @returns {string}
 */
function modifyUrlParam(url, key, value) {
    var pattern = key + '=([^&]*)';
    var replaceText = key + '=' + value;
    if (url.match(pattern)) {
        var tmp = '/(' + key + '=)([^&]*)/gi';
        return url.replace(eval(tmp), replaceText);
    }
    return url.match('[?]') ? url + '&' + replaceText : url + '?' + replaceText;
}

/** @module 浏览器相关-browser */
/**
 * 判断是否是IE浏览器
 *
 * @returns {Boolean}
 */
function isIE() {
    return window['ActiveXObject'] !== 'undefined' || 'ActiveXObject' in window;
}
/**
 * 判断是否是IE11
 *
 * @returns {Boolean}
 */
function isIE11() {
    var ua = navigator.userAgent;
    return ua.indexOf('Trident') > -1 && ua.indexOf('rv:11.0') > -1;
}
/**
 * 判断是否是Edge
 *
 * @returns {Boolean}
 */
function isEdge() {
    return navigator.userAgent.indexOf('Edge') > -1;
}
/**
 * 获取IE浏览器版本
 */
function getIEVersion() {
    var match = navigator.appVersion.match(/MSIE\s+\d+.0;/);
    if (match == null)
        { return false; }
    var result = match[0].match(/\d+/);
    if (result && isArray(result)) {
        return +result[0];
    }
    return null;
}
/**
 * 判断是否是Android
 *
 * @returns {Boolean}
 */
function isAndroid() {
    return /(Android)/i.test(navigator.userAgent);
}
/**
 * 判断是否是IOS
 *
 * @returns {Boolean}
 */
function isIOS() {
    return /(iPhone|iPad|iPod|iOS)/i.test(navigator.userAgent);
}
/**
 * 判断是否是移动端
 *
 * @returns {Boolean}
 */
function isMobile() {
    return isAndroid() || isIOS();
}
/**
 * 判断是否是PC
 *
 * @returns {Boolean}
 */
function isPC() {
    return !isMobile();
}
/**
 * 判断是否是微信浏览器
 *
 * @returns {Boolean}
 */
function isWX() {
    return /micromessenger/.test(navigator.userAgent.toLowerCase());
}

/** @module 浏览器相关-browser */
/**
 *  获取事件源
 *
 * @param {Event} event - 事件源
 */
function getEvent(event) {
    if (event)
        { return event; }
    return window['event'] ? window['event'] : null;
}
/**
 * 获取事件对象
 *
 * @param {Event} event - 事件
 */
function getEventTarget(event) {
    var e = getEvent(event);
    return e.target || e.srcElement;
}
/**
 * 添加事件
 *
 * @param {Element} element - 元素
 * @param {String} type - 事件类型
 * @param {EventListener} handler - 事件
 */
function addHandler(element, type, handler) {
    if (element.addEventListener) {
        element.addEventListener(type, handler, false);
    }
    else if (element['attachEvent']) {
        element['attachEvent']("on" + type, handler);
    }
    else {
        element["on" + type] = handler;
    }
}
/**
 * 移除事件
 *
 * @param {Element} element - 元素
 * @param {String} type - 事件类型
 * @param {EventListener} handler - 事件
 */
function removeHandler(element, type, handler) {
    if (element.removeEventListener) {
        element.removeEventListener(type, handler, false);
    }
    else if (element['detachEvent']) {
        element['detachEvent']("on" + type, handler);
    }
    else {
        element["on" + type] = null;
    }
}
/**
 * 阻止默认事件
 */
function preventDefault(event) {
    var e = getEvent(event);
    if (e.preventDefault) {
        e.preventDefault();
    }
    else {
        e.returnValue = false;
    }
}
/**
 * 阻止事件冒泡
 */
function stopPropagation(event) {
    var e = getEvent(event);
    if (e.stopPropagation) {
        e.stopPropagation();
    }
    else {
        e.cancelBubble = true;
    }
}
/**
 * 获取当前位置的X轴坐标
 *
 * @param {Event} event - 事件源
 */
function getPageX(event) {
    var e = getEvent(event);
    return typeof e.pageX === 'undefined'
        ? e.clientX + (document.body.scrollLeft || document.documentElement.scrollLeft)
        : e.pageX;
}
/**
 * 获取当前位置的Y轴坐标
 *
 * @param {Event} event - 事件源
 */
function getPageY(event) {
    var e = getEvent(event);
    return typeof e.pageY === 'undefined'
        ? e.clientY + (document.body.scrollTop || document.documentElement.scrollTop)
        : e.pageX;
}
/**
 * 获取页面滚动高度
 *
 * @returns {Number}
 */
function getScrollTop() {
    return document.documentElement.scrollTop || window.pageYOffset || document.body.scrollTop;
}

/** @module 函数相关-function */
/**
 * 缓存函数调用结果
 *
 * @param {Function} fn - 函数
 */
function memoize(fn) {
    var cachedArg;
    var cachedResult;
    return function (arg) {
        if (cachedArg === arg)
            { return cachedResult; }
        cachedArg = arg;
        cachedResult = fn(arg);
        return cachedResult;
    };
}

/**
 * 防抖函数
 *
 * @param {Function} fn - 需要防抖处理的函数
 * @param {number} delay - 防抖间隔
 *
 * @returns {Function}
 */
function debounce(fn, delay) {
    var timer;
    return function () {
        var arguments$1 = arguments;

        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments$1[_i];
        }
        clearTimeout(timer);
        timer = setTimeout(function () {
            fn.apply(_this, args);
        }, delay);
    };
}
/**
 * 节流函数
 *
 * @param {Function} fn - 需要节流处理的函数
 * @param {number} threshold - 节流阀值
 *
 * @returns {Function}
 */
function throttle(fn, threshold) {
    var timer;
    var start = +new Date();
    return function () {
        var arguments$1 = arguments;

        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments$1[_i];
        }
        clearTimeout(timer);
        var curr = +new Date() - 0;
        if (curr - start >= threshold) {
            fn.apply(this, args);
            start = curr;
        }
        else {
            timer = setTimeout(function () {
                fn.apply(_this, args);
            }, threshold);
        }
    };
}

export { addClass, addHandler, addUrlParam, camelToKebab, clearStorage, debounce, differenceArr, divideText, flattenArr, formatDate, formatHTML, getAgoDate, getCookie, getCountDown, getDate, getEvent, getEventTarget, getFirstMonday, getFirstSunday, getHighlight, getHighlightList, getHighlightTemplate, getIEVersion, getLastSaturday, getLastSunday, getMonthDays, getMonthLastDay, getNextDate, getNextMonth, getPageX, getPageY, getPrevMonth, getScrollTop, getStorage, getUrlObj, getUrlParam, getWeekFirstDay, getWeekIndex, getWeekLastDay, hasClass, intersectArr, isAndroid, isArray, isEdge, isIE, isIE11, isIOS, isMobile, isPC, isWX, kebabToCamel, mapToObj, memoize, modifyUrlParam, objToMap, prefixDate, preventDefault, randomString, removeClass, removeCookie, removeHandler, removeStorage, setCookie, setStorage, sortJSON, spliceText, stopPropagation, throttle, toDecimal, toggleClass, unionArr, uniqueArr, uniqueKeyInArr, uniqueObjInArr };
